// import PageTitle from '@/components/Dashboard/PageTitle';
import { Button } from '@/components/ui/button';
import { Input } from '@/components/ui/input';
import { Label } from '@/components/ui/label';
import { useFormik, Form, FormikProvider } from 'formik';
import { toast } from 'sonner';
import * as Yup from 'yup';
import { useGetInstDesignersQuery } from '@/app/backend/endpoints/accounts';
import Fallback from '@/components/Fallback';
import { useNavigate, useParams } from 'react-router-dom';
import {
  useCreateMarketCourseMutation,
  useGetMarketCourseByIdQuery,
  useGetTagsQuery,
  useUpdateMarketCourseMutation,
} from '@/app/backend/endpoints/courses';
import ComboBox from '@/components/ComboBox';
import { useEffect, useRef, useState } from 'react';
import { apiUrl } from '@/app/backend';
import { UploadCloudIcon } from 'lucide-react';
import CreatableSelect from 'react-select/creatable';

//   course_id?: number; no field in the form
//   inst_designer_id: number; no field in the form
//   inst_designer_firstName: string; no field in the form
//   inst_designer_lastName: string; no field in the form
//   title: string;
//   description: string;
//   price: number;
//   tags?: string[];
//   videoThumbnail?: string;
//   thumbnail?: string;
//   status: string; no field in the form
//   totalHours?: number;
//   chaptersCount: number;
//   createdAt?: Date; no field in the form
//   updatedAt?: Date; no field in the form
//   type: string; no field in the form

function CreateMarketCourse({ isEdit = false }: { isEdit?: boolean }) {
  const { courseId } = useParams();
  const navigate = useNavigate();
  const { data: courseDataWrap, isLoading: isGetLoading } =
    useGetMarketCourseByIdQuery(parseInt(courseId ?? ''), { skip: !isEdit });
  const courseData = courseDataWrap?.data;
  const { data: instDesignersDataWrap, isLoading: isGetInstDesigners } =
    useGetInstDesignersQuery();
    const [fileInputKey, setFileInputKey] = useState(
      new Date().getTime().toString()
    );
  const instDesignersData = instDesignersDataWrap?.data;
  
  const instDesigners = instDesignersData?.map((instDesigner) => ({
    label: `${instDesigner.firstName} ${instDesigner.lastName}`,
    value: instDesigner.user_id.toString(),
  }));

  const [isLoadingImage, setIsLoadingImage] = useState(false);

  const [createCourse, { isLoading }] = useCreateMarketCourseMutation();
  const [updateCourse, { isLoading: isUpdating }] =
    useUpdateMarketCourseMutation();
  const fileInputRef = useRef<HTMLInputElement>(null);
  const handleButtonClick = (e: React.MouseEvent<HTMLButtonElement>) => {
    e.preventDefault();
    fileInputRef.current?.click();
  };

  const { data: tagsDataWrap, isLoading: isGetTags } = useGetTagsQuery();
  const tagsData = tagsDataWrap?.data;
  const [tags, setTags] = useState<
    {
      label: string;
      value: string;
    }[]
  >([]);
  useEffect(() => {
    if (tagsData) {
      setTags(
        tagsData?.map((tag) => ({
          label: tag,
          value: tag,
        }))
      );
    }
  }, [tagsData]);

  const formik = useFormik<CEMarketCourseI>({
    initialValues: {
      inst_designer_id: courseData?.inst_designer_id.toString() ?? '',
      inst_designer_firstName: courseData?.inst_designer_firstName ?? '',
      inst_designer_lastName: courseData?.inst_designer_lastName ?? '',
      title: courseData?.title ?? '',
      description: courseData?.description ?? '',
      price: courseData?.price ?? 0,
      tags: courseData?.tags ?? [],
      videoThumbnail: courseData?.videoThumbnail ?? '',
      thumbnail: courseData?.thumbnail ?? '',
      totalHours: courseData?.totalHours ?? 0,
      chaptersCount: courseData?.chaptersCount ?? 0,
      status: courseData?.status ?? 'active',
      type: 'market',
    },
    validationSchema: Yup.object().shape({
      title: Yup.string().required('Title is required'),
      description: Yup.string().required('Description is required'),
      price: Yup.number()
        .min(1, 'Price is required')
        .required('Price is required'),
      totalHours: Yup.number()
        .min(1, 'Total hours is required')
        .required('Total hours is required'),
      chaptersCount: Yup.number()
        .min(1, 'Chapters count is required')
        .required('Chapters count is required'),
      inst_designer_id: Yup.string().required(
        'Instructor designer is required'
      ),
      thumbnail: Yup.string()
        .min(1, 'Thumbnail is required')
        .required('Thumbnail is required'),
    }),
    enableReinitialize: true,
    validateOnMount: false,
    validateOnBlur: false,
    validateOnChange: false,
    onSubmit: (course: CEMarketCourseI) => {
      if (isEdit) {
        updateCourse({ course_id: Number(courseId), ...course })
          .unwrap()
          .then((response) => {
            toast.success(response.message);
            navigate('/dashboard/market-courses/list');
          })
          .catch((error: any) => {
            toast.error(error?.data?.message ?? error?.error.toString());
          });
      } else {
        createCourse(course)
          .unwrap()
          .then((response) => {
            toast.success(response.message);
            navigate('/dashboard/market-courses/list');
          })
          .catch((error: any) => {
            toast.error(error?.data?.message ?? error?.error.toString());
          });
      }
    },
  });
  const {
    getFieldProps,
    handleSubmit,
    touched,
    errors,
    setFieldValue,
    values,
  } = formik;
  const handleFileChange = (e: any) => {
    const file = e.target.files?.[0];
    if (file) {
      const reader = new FileReader();
      reader.onload = () => {
        if (reader.readyState === 2) {
          // upload file to server
          const formData = new FormData();
          setIsLoadingImage(true);
          formData.append('file', file);
          fetch(`${apiUrl}/file/thumbnails/upload`, {
            method: 'POST',
            body: formData,
            credentials: 'include',
          })
            .then((res) => res.json())
            .then((response) => {
              // console.log(response.data.fileUrl);

              setFieldValue('thumbnail', response.data.fileUrl);
              setIsLoadingImage(false);
            })
            .catch(() => {
              toast.error('Error uploading thumbnail');
              setIsLoadingImage(false);
            });
        }
      };
      reader.readAsDataURL(file);
    }
  };

  const handleDeleteThumbnail = (e: React.MouseEvent<HTMLButtonElement>) => {
    e.preventDefault();
    setIsLoadingImage(true);
    fetch(`${apiUrl}/file/thumbnails/delete?file=${values.thumbnail}`, {
      method: 'DELETE',
      headers: {
        'Content-Type': 'application/json',
      },
      credentials: 'include',
    })
      .then(() => {
        setFileInputKey(new Date().getTime().toString());
        setFieldValue('thumbnail', '');
        setIsLoadingImage(false);
      })
      .catch(() => {
        setFileInputKey(new Date().getTime().toString());
        setFieldValue('thumbnail', '');
        setIsLoadingImage(false);
      });
  };

  if (isEdit && (isGetLoading || isGetInstDesigners || isGetTags))
    return <Fallback />;
  return (
    <>
      {/* <PageTitle title="Create User" /> */}
      <div className="flex justify-center items-center">
        <div className="w-full px-6">
          <FormikProvider value={formik}>
            <Form onSubmit={handleSubmit}>
              <div className="flex justify-between items-center gap-4 py-2">
                <span className="text-slate-600 text-[25px] font-semibold">
                  {isEdit ? 'Edit' : 'Create'} Course
                </span>
                <div className="space-x-2">
                  <Button
                    variant="outline"
                    className="font-semibold font-inter rounded-md"
                    type="button"
                    onClick={(e) => {
                      const res = window.confirm(
                        'Are you sure you want to cancel?'
                      );
                      if (!isEdit && !res) {
                        return;
                      }
                      if (!isEdit && res && values.thumbnail) {
                        handleDeleteThumbnail(e);
                      }
                      navigate('/dashboard/market-courses/list');
                    }}
                  >
                    Cancel
                  </Button>
                  <Button
                    type="submit"
                    className="bg-amber-500 rounded-md font-semibold font-inter text-white"
                    isLoading={
                      isLoading ||
                      isGetInstDesigners ||
                      isGetLoading ||
                      isUpdating
                    }
                    disabled={isLoadingImage}
                  >
                    Save
                  </Button>
                </div>
              </div>
              <div className="grid grid-cols-2 gap-y-4 gap-x-6 py-5">
                {inputs.map(
                  ({ name, type = 'text', label, required, ...props }, i) => {
                    // if (isEdit && name === 'password') return null;
                    return (
                      <div className="grid gap-2" key={i}>
                        <div className="flex items-center space-x-1">
                          <Label htmlFor={name}>{label}</Label>
                          {required! && <span className="text-red-500">*</span>}
                        </div>
                        {type !== 'comboBox' &&
                        type !== 'image' &&
                        type !== 'multiSelect' ? (
                          <Input
                            className="bg-white"
                            type={type}
                            {...props}
                            {...getFieldProps(name)}
                            error={
                              touched[name] && errors[name]
                                ? errors[name]
                                : undefined
                            }
                          />
                        ) : type !== 'image' && type !== 'multiSelect' ? (
                          <ComboBox
                            defaultText="Select a designer"
                            data={instDesigners!}
                            value={values?.inst_designer_id?.toString() ?? ''}
                            setValue={(value) => {
                              setFieldValue('inst_designer_id', value);
                              setFieldValue(
                                'inst_designer_firstName',
                                instDesignersData?.find(
                                  (instDesigner) =>
                                    instDesigner.user_id === parseInt(value)
                                )?.firstName ?? ''
                              );
                              setFieldValue(
                                'inst_designer_lastName',
                                instDesignersData?.find(
                                  (instDesigner) =>
                                    instDesigner.user_id === parseInt(value)
                                )?.lastName ?? ''
                              );
                            }}
                            error={
                              touched[name] && errors[name]
                                ? errors[name]
                                : undefined
                            }
                          />
                        ) : type !== 'multiSelect' ? (
                          <div className="space-y-1 flex flex-col">
                            <input
                              key={fileInputKey}
                              type="file"
                              accept="image/*"
                              onChange={handleFileChange}
                              ref={fileInputRef}
                              className="hidden"
                            />
                            {touched[name] && errors[name] && (
                              <p className="text-red-500 text-sm">
                                Thumbnail is required
                              </p>
                            )}
                            {values.thumbnail ? (
                              <>
                                <Button
                                  className="ml-1 w-44 bg-red-500 hover:bg-red-600 text-white font-semibold rounded-md"
                                  onClick={handleDeleteThumbnail}
                                  isLoading={isLoadingImage}
                                >
                                  Delete
                                </Button>
                                <img
                                  draggable={false}
                                  src={values.thumbnail}
                                  alt="thumbnail"
                                  className="max-w-[700px] object-cover rounded-md"
                                />
                              </>
                            ) : (
                              <Button
                                onClick={handleButtonClick}
                                isLoading={isLoadingImage}
                                className="w-44 bg-amber-50 hover:bg-amber-100 hover:border-2 text-amber-800 rounded-md font-semibold font-inter border border-amber-100"
                                loadingSpinnerColor="amber-800"
                              >
                                <UploadCloudIcon className="h-5 w-5 mr-2" />
                                Upload an Image
                              </Button>
                            )}
                          </div>
                        ) : (
                          <CreatableSelect
                            value={values?.tags?.map((tag) => ({
                              label: tag,
                              value: tag,
                            }))}
                            isMulti
                            options={tags}
                            onChange={(value) => {
                              setFieldValue(
                                'tags',
                                value.map((v) => v.value)
                              );
                            }}
                          />
                        )}
                      </div>
                    );
                  }
                )}
              </div>
              {/* {!isLoading &&
              !isGetInstDesigners &&
              !isGetLoading &&
              !isUpdating ? (
                <Button type="submit" className="w-full rounded-sm">
                  {isEdit ? 'Update' : 'Create'}
                </Button>
              ) : (
                <Button
                  disabled
                  type="submit"
                  className="w-full text-white focus:ring-4 font-medium rounded-sm text-sm px-5 py-2.5 text-center inline-flex items-center"
                >
                  <svg
                    aria-hidden="true"
                    role="status"
                    className="inline w-4 h-4 me-3 text-white animate-spin"
                    viewBox="0 0 100 101"
                    fill="none"
                    xmlns="http://www.w3.org/2000/svg"
                  >
                    <path
                      d="M100 50.5908C100 78.2051 77.6142 100.591 50 100.591C22.3858 100.591 0 78.2051 0 50.5908C0 22.9766 22.3858 0.59082 50 0.59082C77.6142 0.59082 100 22.9766 100 50.5908ZM9.08144 50.5908C9.08144 73.1895 27.4013 91.5094 50 91.5094C72.5987 91.5094 90.9186 73.1895 90.9186 50.5908C90.9186 27.9921 72.5987 9.67226 50 9.67226C27.4013 9.67226 9.08144 27.9921 9.08144 50.5908Z"
                      fill="#E5E7EB"
                    />
                    <path
                      d="M93.9676 39.0409C96.393 38.4038 97.8624 35.9116 97.0079 33.5539C95.2932 28.8227 92.871 24.3692 89.8167 20.348C85.8452 15.1192 80.8826 10.7238 75.2124 7.41289C69.5422 4.10194 63.2754 1.94025 56.7698 1.05124C51.7666 0.367541 46.6976 0.446843 41.7345 1.27873C39.2613 1.69328 37.813 4.19778 38.4501 6.62326C39.0873 9.04874 41.5694 10.4717 44.0505 10.1071C47.8511 9.54855 51.7191 9.52689 55.5402 10.0491C60.8642 10.7766 65.9928 12.5457 70.6331 15.2552C75.2735 17.9648 79.3347 21.5619 82.5849 25.841C84.9175 28.9121 86.7997 32.2913 88.1811 35.8758C89.083 38.2158 91.5421 39.6781 93.9676 39.0409Z"
                      fill="currentColor"
                    />
                  </svg>
                  Loading...
                </Button>
              )} */}
            </Form>
          </FormikProvider>
        </div>
      </div>
    </>
  );
}
const inputs: InputRequiredFields<CEMarketCourseI>[] = [
  {
    required: true,
    name: 'title',
    placeholder: 'Enter the course title',
    type: 'text',
    id: 'title',
    label: 'Title',
    autoFocus: true,
  },
  {
    required: true,
    name: 'description',
    placeholder: 'Enter the course description',
    type: 'text',
    id: 'description',
    label: 'Description',
  },
  {
    required: true,
    name: 'price',
    placeholder: 'Enter the course price',
    type: 'number',
    id: 'price',
    label: 'Price',
  },
  {
    required: false,
    name: 'tags',
    placeholder: 'Enter the course tags',
    type: 'multiSelect',
    id: 'tags',
    label: 'Tags',
  },
  {
    required: false,
    name: 'videoThumbnail',
    placeholder: 'Enter youtube link',
    type: 'text',
    id: 'videoThumbnail',
    label: 'Video Thumbnail',
  },
  {
    required: true,
    name: 'totalHours',
    placeholder: 'Enter the course total hours',
    type: 'number',
    id: 'totalHours',
    label: 'Total Hours',
  },
  {
    required: true,
    name: 'chaptersCount',
    placeholder: 'Enter the course chapters count',
    type: 'number',
    id: 'chaptersCount',
    label: 'Chapters Count',
  },
  {
    required: true,
    name: 'inst_designer_id',
    placeholder: 'Select a designer',
    type: 'comboBox',
    id: 'inst_designer_id',
    label: 'Instructor Designer',
  },
  {
    required: true,
    name: 'thumbnail',
    placeholder: 'Upload a thumbnail',
    type: 'image',
    id: 'thumbnail',
    label: 'Thumbnail',
  },
];
export default CreateMarketCourse;
